/**
 * @file popup
 * @author songjunke & luowenwen02
 * @since 2018-08-21
 */

/* global chrome, Semanticization, HolmesServices */
/* disable-eslint max-len */

const VERSION = '0.0.3';

let utils = {
    isEmptyObject: obj => Object.keys(obj).length === 0,
    warning: content => `<span class="error">${content}</span>`
};

const UNLOGIN = 2;
const ERROR = 1;
const SUCCECSS = 0;

$(document).ready(function () {
    const semantics = new Semanticization();

    let uaType = 'pc';
    // 获取 background 的 window 对象
    let background = chrome.extension.getBackgroundPage();

    let tabs1 = {
        tabId: null,
        lastsSiList: {},
        isEmpty: false,
        reset() {
            $('.detail', '#tabs-1').empty();
            this.sListInfo = [];
            this.isEmpty = true;
            this.lastsSiList = {};
        },
        hideLoading() {
            $('.message', '#tabs-1').fadeIn();
            $('.loading', '#tabs-1').hide();
        },
        showLoading() {
            $('.message', '#tabs-1').hide();
            $('.loading', '#tabs-1').show();
        },
        show(tab, message, appendItem) {
            this.showLoginTip();
            this.isEmpty = false;
            if (appendItem) {
                let requestData;
                if (appendItem.type === 'j') {
                    requestData = {
                        [appendItem.si]: {
                            j: [appendItem],
                            g: []

                        }
                    };
                }
                else {
                    requestData = {
                        [appendItem.si]: {
                            g: [appendItem],
                            j: []
                        }
                    };
                }
                this.convertSiList(requestData);
                return;
            }

            this.showLoading();

            let tabId = tab[0].id;
            this.tabId = tabId;
            let curTabData = message[tabId];
            if (curTabData && !utils.isEmptyObject(curTabData.data)) {
                if (curTabData.result) {
                    this.showSuccess(curTabData.result);

                    return;
                }
                let requestData = curTabData.data;
                this.convertSiList(requestData);
            }
            else {
                this.showFailure();
            }
        },
        showSuccess(data) {
            this.sListInfo = data;
            data.forEach(sInfo => {
                this.appendReqItem(sInfo);
            });
            this.hideLoading();
        },
        showFailure() {
            let text = '';

            if (!navigator.onLine) {
                text = utils.warning('网络已断开，请稍后重试');
            }
            else {
                text = utils.warning('未检测到统计代码，如果该页面加载于插件安装之前，请尝试刷新页面后再重新执行检查');
            }

            $('.detail', '#tabs-1').html(text);

            this.hideLoginTip();
            this.hideLoading();
        },
        showLogin(data) {
            $('.detail', '#tabs-1').html(
                '未登录百度统计，请<a href="https://tongji.baidu.com/web/welcome/login?loginDialog=1" class="login-link" target="_blank">登录</a>'
            );

            this.hideLoginTip();
            this.hideLoading();
        },

        showLoginTip() {
            $('.login-tip').show();
        },

        hideLoginTip() {
            $('.login-tip').hide();
        },

        appendReqItem(sInfo) {
            let text;
            let errorMsg;
            let duplicateMsg = sInfo.isDuplicate ? '重复' : '';
            if (sInfo.isCorrect) {
                text = `<div class="d-item">您${duplicateMsg}安装了账户"${sInfo.account}"的站点"${sInfo.domain}"的统计代码；`;
            }
            // 说明没有经过web判断
            else if (!sInfo.account) {
                if (!sInfo.referer) {
                    errorMsg = '<li class="error">•&nbsp;该页面禁用了referer</li>';
                }
                else {
                    errorMsg = '<li class="error">•&nbsp;当前配置文件可能还未生效</li>';
                }
                text = `<div class="d-item">
                            <div>您错误安装了站点id为 <b>${sInfo.si}</b> 的统计代码，错误原因如下：</div>
                            <ul>${errorMsg}</ul>
                        </div>`;
            }
            else {
                text = `<div class="d-item">该页面${duplicateMsg}安装了不属于您账号的统计代码<br/>`
                    + `(站点id:${sInfo.si};所属站点:${sInfo.domain};所属账号:${sInfo.account})`;
            }
            $('.detail', '#tabs-1').append(text);
            background.cacheResult(this.tabId, $.extend(true, [], this.sListInfo));
            this.hideLoading();
        },
        getAccountInfoBySi(si) {
            return new Promise((resolve, reject) => {
                this.lastsSiList[si] = this.lastsSiList[si] ? this.lastsSiList[si] : {value: null};
                if (this.lastsSiList[si].value) {
                    resolve(this.lastsSiList[si].value);
                    return;
                }
                background.getCheckResultBySiList({siList: [si]}, (data, status) => {
                    if (status === SUCCECSS) {
                        let correct = data.data.correct;
                        let incorrect = data.data.incorrect;
                        let itemInfo;

                        correct.some(item => {
                            if (item.si === si) {
                                itemInfo = item;
                                itemInfo.isCorrect = true;
                                return true;
                            }
                            else {
                                return false;
                            }
                        });
                        if (!itemInfo) {
                            incorrect.some(item => {
                                if (item.si === si) {
                                    itemInfo = item;
                                    itemInfo.isCorrect = false;
                                    return true;
                                }
                                else {
                                    return false;
                                }
                            });
                        }

                        this.lastsSiList[si].value = itemInfo;
                        resolve(itemInfo);
                        return;
                    }
                    reject(status);
                });
            });
        },
        sListInfo: [],
        isDuplicate(si) {
            return this.lastsSiList[si];
        },
        formatRequestItem(request) {
            return new Promise((resolve, reject) => {
                let requestItem = $.extend(true, {}, request);
                let si = requestItem.si;
                let isDuplicate = this.isDuplicate(si);

                // js
                if (requestItem.type === 'j') {
                    requestItem.isDuplicate = isDuplicate;
                    if (!requestItem.contentLength > 0) {
                        requestItem.isCorrect = false;
                        resolve(requestItem);
                    }
                    else {
                        this.getAccountInfoBySi(si).then(accountInfo => {
                            Object.assign(requestItem, accountInfo);
                            resolve(requestItem);
                        }).catch(err => {
                            reject(err);
                        });
                    }
                }
                // gif
                else {
                    // gif重复的不用管
                    if (isDuplicate) {
                        resolve(null);
                        return;
                    }

                    this.getAccountInfoBySi(si).then(accountInfo => {
                        Object.assign(requestItem, accountInfo);
                        resolve(requestItem);
                    }).catch(err => {
                        reject(err);
                    });
                }
            });
        },
        convertSiList(requestData) {
            // 先自行判断一遍
            for (let i in requestData) {
                let hmInfo = requestData[i];

                // 先判断是否存在js
                if (hmInfo.j.length) {
                    hmInfo.j.forEach(item => {
                        this.formatRequestItem(item).then(formatItem => {
                            if (formatItem) {
                                this.sListInfo.push(formatItem);
                                this.appendReqItem(formatItem);
                            }
                        }).catch(err => {
                            if (err === UNLOGIN) {
                                this.showLogin();
                            }
                        });
                    });
                }
                // 再判断是否存在gif
                else if (hmInfo.g.length) {
                    let gList = hmInfo.g.filter(function (item) {
                        return !HolmesServices.isUnsentLog(item);
                    });

                    if (!gList.length) {
                        continue;
                    }
                    // 就拿第一个吧
                    this.formatRequestItem(hmInfo.g[0]).then(formatItem => {
                        if (formatItem) {
                            this.sListInfo.push(formatItem);
                            this.appendReqItem(formatItem);
                        }
                    }).catch(err => {
                        if (err === UNLOGIN) {
                            this.showLogin();
                        }
                    });
                }

            }
        }

    };

    let tabs2 = {
        count: 0,
        isRendered: false,
        timer2: null,
        reset() {
            this.isEmpty = true;
            let $elm = $('#accordion');
            $elm.empty();
            this.count = 0;
            this.isRendered = false;
        },
        hideLoading() {
            $('#accordion').fadeIn();
            $('#tabs-2 .loading').hide();
        },
        showLoading() {
            $('#accordion').hide();
            $('.loading', '#tabs-2 ').show();
        },
        appendReqItem(item) {
            this.count++;
            let $accordion = $('#accordion');

            const logTypeName = semantics.getEventType(item);

            if (logTypeName) {
                const eventInfo = semantics.getEventDescription(item);

                let html = `
                <h3 class="ellipsis">${logTypeName}</h3>
                <div id="accordion-${this.count}"></div>
                `;
                $accordion.append(html);

                let eventInfoLabel = [];
                for (let i = 0; i < eventInfo.length; i++) {
                    eventInfoLabel.push('<p>' + eventInfo[i] + '</p>');
                }

                // 加上 siteid 的签名
                eventInfoLabel.push('<p>统计JS代码为：' + item.si + '</p>');

                $('#accordion-' + this.count).html(eventInfoLabel.join(''));

                $('#accordion').accordion('refresh');

                $('.ui-accordion-header', '#accordion').off('click').click(function () {
                    $(this).toggleClass('ui-state-active');
                    $(this).next().toggle();
                });
            }
        },
        isEmpty: false,
        show(tab, message, appendItem) {
            this.isEmpty = false;

            if (this.isRendered && !appendItem) {
                return;
            }
            let $elm = $('#accordion');
            if (appendItem) {
                if (!this.initAccordion) {
                    // 初始化 accordion
                    $elm.empty();
                    $elm.accordion({
                        collapsible: true,
                        animate: false,
                        icons: false,
                        heightStyle: 'content',
                        active: false
                    });

                    this.initAccordion = true;
                }
                if (semantics.getEventType(appendItem) && !HolmesServices.isUnsentLog(appendItem)) {
                    this.appendReqItem(appendItem);
                }
                return;
            }

            this.showLoading();
            let tabId = tab[0].id;
            let curTabData = message[tabId];
            // 过滤出所有的gif请求
            if (curTabData) {
                let requestData = curTabData.data;

                // 初始化 accordion
                $elm.empty();
                $elm.accordion({
                    collapsible: true,
                    animate: false,
                    icons: false,
                    heightStyle: 'content',
                    active: false
                });

                this.initAccordion = true;

                for (let i in requestData) {
                    let hmInfo = requestData[i];

                    if (hmInfo.g.length) {
                        hmInfo.g.forEach(item => {
                            // 目前能识别的，并且非补发请求
                            if (semantics.getEventType(item) && !HolmesServices.isUnsentLog(item)) {
                                this.appendReqItem(item);
                            }
                        });
                    }

                }
            }
            else {
                this.showFailure();
            }

            this.hideLoading();
            this.isRendered = true;
        },
        showFailure() {
            $('#accordion').html('<span class="error">尚未产生服务器请求，如果该页面加载于百度统计助手安装之前，请尝试刷新后再重新执行监听</span>');
        }
    };

    // 处理当前tab
    let handleCurTab = (cb, ...params) => {
        chrome.windows.getCurrent(function (window) {
            let wid = window.id;
            let context = params.pop();

            chrome.tabs.query({active: true, windowId: wid}, function (tab) {
                cb.call(context, tab, ...params);
            });
        });
    };

    let init = function () {
        // 获取后台数据
        let rawData = background.getMetaData();
        handleCurTab(tabs1.show, rawData, tabs1);
    };
    init();

    // 向content-script发送消息
    let sendMessageToContentScript = (tab, ...params) => {
        let tabId = tab[0].id;
        let message = params[0];
        let cb = params[1];

        chrome.tabs.sendMessage(tabId, message, function (response) {
            cb && cb(response);
        });
    };

    let listenHandler = (message, sender, sendResponse) => {
        if (message.type === 'RECEIVEDREQUEST') {
            let tabIndex = $('#tabs').tabs('option', 'active');
            let data = JSON.parse(message.data);

            if (data.type === 'g' && tabIndex === 1) {
                handleCurTab(tabs2.show, background.getMetaData(), data, tabs2);
            }

            handleCurTab(tabs1.show, background.getMetaData(), data, tabs1);
        }
        if (message.type === 'RESET') {
            tabs2.reset();
            tabs1.reset();
        }

        if (message.type === 'REFRESH') {
            let tabIndex = $('#tabs').tabs('option', 'active');

            if (tabs1.isEmpty) {
                handleCurTab(tabs1.showFailure, background.getMetaData(), tabs1);
            }
            if (tabs2.isEmpty && tabIndex === 1) {
                handleCurTab(tabs2.showFailure, background.getMetaData(), tabs2);
            }
        }
    };

    // 监听事件
    chrome.runtime.onMessage.addListener(listenHandler);
    // 初始化 tabs
    $('#tabs').tabs({
        activate(event, ui) {
            if (ui.newTab.is('#tabs2-li')) {
                handleCurTab(tabs2.show, background.getMetaData(), tabs2);
            }
        }
    });

    (function bindEvents() {
        let $select = $('.ua-select');
        uaType = background.getUaType();
        $select.find('option[value=' + uaType + ']').attr('selected', true);

        $select.on('change.select', function (e) {
            let uaType = $.trim($(this).val());

            handleCurTab(sendMessageToContentScript, {uaType: uaType, type: 'SETUA'}, this);
        });

        $('.login-link').on('click', function () {
            handleCurTab(function (tab) {
                background.resetCacheResult(tab[0].id);
            }, this);
        });
    })();


    // 版本检查
    HolmesServices.checkoutVersion(VERSION, function (value) {
        if (value === -1) {
            let versionHtml = `
                <div class="version-tip-wrapper">
                    <div class="version-tip">
                        <div class="text">版本更新了，点击<a href="https://tongji.baidu.com/static/tongji/hm-debugger/upgrade.html" target="_blank">此处</a>下载安装吧</div>
                    </div>
                </div>
            `;
            $('.main').prepend(versionHtml);
        }
    });
});
