﻿/**
 * @file Semanticization
 * @author wumingdan
 * @since 2018-08-23
 */

/* global HolmesServices */

class Semanticization {

    constructor() {
        this.eventMap = {
            0: 'PV日志',
            1: '事件转化',
            2: '页面点击图',
            3: '页面访问时长',
            7: 'JS API 自定义事件(_trackCustomEvent)',
            8: 'JS API 业务ID设置(_setUserId)',
            4: 'JS API 事件跟踪(_trackEvent)',
            94: 'JS API 订单请求(_trackOrder)'
        };

        this.customTagMap = {
            1: '用户标签',
            2: '会话标签',
            3: '页面标签'
        };
    }

    getEventType(log) {
        const et = log ? log.et : undefined;

        return this.eventMap[et];
    }

    getEventDescription(log) {
        const et = log ? log.et : undefined;

        if (this['parseEventType' + et]) {
            const epInfo = this['parseEventType' + et](log);

            return epInfo;
        }

        return [];
    }

    parseEventType0(log) {
        // tt & isTrackPageview & v（pixel） & ct & cv
        let result = [];

        // 1. tt
        const tt = log.tt;
        tt && result.push('页面标题：' + HolmesServices.encodeHTML(tt));

        // 2. isTrackPageview
        if (HolmesServices.isJSAPILog(log) && typeof HolmesServices.encodeHTML(log.u) !== 'undefined') {
            result.push('PV 信息：由 _trackPageview 发出，页面 URL 为：' + HolmesServices.encodeHTML(log.u));
        }

        // 3. v
        const v = log.v;
        if (/pixel/.test(v)) {
            result.push('来自 pixel 代码的 PV 事件');
        }

        // 4. ct
        const ct = log.ct;
        if (ct) {
            const tags = ct.split('!');

            tags.forEach((tag, index) => {
                String(tag).split('|').forEach((t) => {
                    const tagInfo = t.split('*');

                    const tagName = this.customTagMap[tagInfo[1]];

                    if (tagName && tagName !== '' && tagInfo[2] !== '') {
                        result.push([tagName, HolmesServices.encodeHTML(tagInfo[2])].join('：'));
                    }
                });
            });
        }

        // 5. cv
        const cv = log.cv;
        if (cv) {
            const cvInfo = cv.split('*');

            if (cvInfo[0] !== undefined) {
                result.push([
                    '自定义变量：index(', HolmesServices.encodeHTML(cvInfo[0]),
                    '), name(', HolmesServices.encodeHTML(cvInfo[1]),
                    '), value(', HolmesServices.encodeHTML(cvInfo[2]), ')'
                ].join(''));
            }
        }

        // 6. p
        const p = log.p;
        if (p) {
            const pInfo = p.split('!');
            let propertyItem;

            pInfo.forEach(item => {
                propertyItem = String(item).split('*');

                if (propertyItem.length > 1) {
                    if (propertyItem[2] && propertyItem[2] === '1') {
                        result.push(`用户属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                    else if (propertyItem[2] && propertyItem[2] === '2') {
                        result.push(`访次属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                    else {
                        result.push(`PV 属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                }
            });
        }

        return result;
    }

    parseEventType1(log) {
        // ep -> event id
        const ep = log.ep;

        let result = [];

        if (ep !== undefined) {
            const idReg = /{id:(.*),eventType/;

            if (idReg.test(ep)) {
                result.push('发生事件转化目标的元素是：' + RegExp.$1);
            }
        }

        return result;
    }

    parseEventType2(log) {
        // heatmap
        const clicks = String(log.ep).split('!');

        return clicks.map((click) => {
            return '发生元素点击：' + click.split('*')[5];
        });
    }

    parseEventType3(log) {
        let result = [];

        // ep
        const epInfo = String(log.ep).split(',');

        result.push(['页面访问时长：' + HolmesServices.formatTime(epInfo[0] / 1000)]);
        // result.push(['页面焦点时长：' + HolmesServices.formatTime(epInfo[1] / 1000)]);

        return result;
    }

    parseEventType4(log) {
        // trackEvent -> category, action, label
        const epInfo = String(log.ep).split('*');

        const errorMsg = '<span class="error"><b>错误的参数设置</b></span>';
        const notSetMsg = '<b>未设置</b>';

        const category = epInfo.length > 0 ? HolmesServices.encodeHTML(epInfo[0]) : errorMsg;
        const action = epInfo.length > 1 ? HolmesServices.encodeHTML(epInfo[1]) : errorMsg;
        const label = epInfo.length > 2 ? HolmesServices.encodeHTML(epInfo[2]) : notSetMsg;

        return [`发生 _trackEvent 事件：category(${category}), action(${action}), label(${label})`];
    }

    parseEventType7(log) {
        const pInfo = String(log.p).split('!');

        let result = [];
        let propertyItem;

        pInfo.forEach(item => {
            propertyItem = String(item).split('*');

            if (propertyItem.length > 1) {
                if (propertyItem[0] === '_iden') {
                    result.unshift(`自定义事件：${propertyItem[1]}`);
                }
                else {
                    if (propertyItem[2] && propertyItem[2] === '1') {
                        result.push(`用户属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                    else if (propertyItem[2] && propertyItem[2] === '2') {
                        result.push(`访次属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                    else {
                        result.push(`事件属性：${propertyItem[0]}(${propertyItem[1]})`);
                    }
                }
            }
        });

        return result;
    }

    parseEventType8(log) {
        return [`业务ID设置：${String(log.p).split('*')[1]}`];
    }

    parseEventType94(log) {
        // order detail
        const ep = log.ep;

        return ['订单参数：' + JSON.stringify(ep)];
    }

}
